# Generated by Django 3.0.6 on 2020-05-09 22:18

import django.db.models.deletion
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('tesla_ce', '0002_histograms'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProviderNotification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(help_text='Notification unique key for the provider', max_length=250)),
                ('info', models.TextField(help_text='Notification information.', null=True)),
                ('when', models.DateTimeField(help_text='When the notification should be sent to provider.')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('provider', models.ForeignKey(help_text='Related provider', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.Provider')),
            ],
            options={
                'unique_together': {('provider', 'key')},
            },
        ),
    ]
