# Generated by Django 3.2.4 on 2021-07-16 08:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tesla_ce', '0017_auto_20210715_2150'),
    ]

    operations = [
        migrations.AddField(
            model_name='assessmentsession',
            name='auto_closed',
            field=models.BooleanField(default=None, help_text='Whether this session has been automatically closed', null=True),
        ),
    ]
