# Generated by Django 3.0.8 on 2020-07-12 11:30

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('tesla_ce', '0006_auto_20200520_2128'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='enrolment',
            name='model_samples',
        ),
        migrations.AddField(
            model_name='enrolment',
            name='model_samples',
            field=models.ManyToManyField(help_text='Enrolment samples used to create this model', related_name='models', to='tesla_ce.EnrolmentSample'),
        ),
    ]
