# Generated by Django 3.2.4 on 2021-10-09 00:08

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tesla_ce', '0020_institution_allowed_domains'),
    ]

    operations = [
        migrations.AlterField(
            model_name='reportactivitysession',
            name='session',
            field=models.OneToOneField(help_text='Assessment Session related to this report.', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.assessmentsession'),
        ),
    ]
