# Generated by Django 3.0.6 on 2020-05-20 19:28

import django.db.models.deletion
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('tesla_ce', '0005_auto_20200520_2009'),
    ]

    operations = [
        migrations.AlterField(
            model_name='assessmentsessiondata',
            name='session',
            field=models.OneToOneField(help_text='Related assessment session', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.AssessmentSession'),
        ),
    ]
