# Generated by Django 3.2.4 on 2021-07-15 19:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tesla_ce', '0016_auto_20210709_1023'),
    ]

    operations = [
        migrations.AddField(
            model_name='reportactivity',
            name='data',
            field=models.FileField(help_text='Path to the content of this report.', max_length=250, null=True, upload_to=''),
        ),
        migrations.CreateModel(
            name='ReportActivitySession',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identity_level', models.SmallIntegerField(choices=[(0, 'Pending'), (1, 'No Information'), (2, 'Ok'), (3, 'Warning'), (4, 'Alert')], default=0, help_text='Alert level for learner identity.')),
                ('content_level', models.SmallIntegerField(choices=[(0, 'Pending'), (1, 'No Information'), (2, 'Ok'), (3, 'Warning'), (4, 'Alert')], default=0, help_text='Alert level for content authorship.')),
                ('integrity_level', models.SmallIntegerField(choices=[(0, 'Pending'), (1, 'No Information'), (2, 'Ok'), (3, 'Warning'), (4, 'Alert')], default=0, help_text='Alert level for system integrity.')),
                ('total_requests', models.IntegerField(default=0, help_text='Number of requests on this session.')),
                ('pending_requests', models.IntegerField(default=0, help_text='Number of pending requests on this session.')),
                ('valid_requests', models.IntegerField(default=0, help_text='Number of valid requests on this session.')),
                ('processed_requests', models.IntegerField(default=0, help_text='Number of processed requests on this session.')),
                ('data', models.FileField(default=None, help_text='Path to the content of this report.', max_length=250, null=True, upload_to='')),
                ('closed_at', models.DateTimeField(default=None, help_text='When the session report was closed', null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('report', models.ForeignKey(help_text='Related Activity Report.', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.reportactivity')),
                ('session', models.ForeignKey(help_text='Assessment Session related to this report.', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.assessmentsession', unique=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
