# Generated by Django 3.0.6 on 2020-05-06 11:42

import django.db.models.deletion
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('tesla_ce', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='HistogramLearnerProvider',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('b0', models.BigIntegerField(default=0, help_text='Bin containing values from 0 to 9%')),
                ('b1', models.BigIntegerField(default=0, help_text='Bin containing values from 10% to 19%')),
                ('b2', models.BigIntegerField(default=0, help_text='Bin containing values from 20% to 29%')),
                ('b3', models.BigIntegerField(default=0, help_text='Bin containing values from 30% to 39%')),
                ('b4', models.BigIntegerField(default=0, help_text='Bin containing values from 40% to 49%')),
                ('b5', models.BigIntegerField(default=0, help_text='Bin containing values from 50% to 59%')),
                ('b6', models.BigIntegerField(default=0, help_text='Bin containing values from 60% to 69%')),
                ('b7', models.BigIntegerField(default=0, help_text='Bin containing values from 70% to 79%')),
                ('b8', models.BigIntegerField(default=0, help_text='Bin containing values from 80% to 89%')),
                ('b9', models.BigIntegerField(default=0, help_text='Bin containing values from 90% to 100%')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('learner', models.ForeignKey(help_text='Related Learner.', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.Learner')),
                ('provider', models.ForeignKey(help_text='Related Provider.', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.Provider')),
            ],
            options={
                'unique_together': {('learner', 'provider')},
            },
        ),
        migrations.CreateModel(
            name='HistogramLearnerInstrument',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('b0', models.BigIntegerField(default=0, help_text='Bin containing values from 0 to 9%')),
                ('b1', models.BigIntegerField(default=0, help_text='Bin containing values from 10% to 19%')),
                ('b2', models.BigIntegerField(default=0, help_text='Bin containing values from 20% to 29%')),
                ('b3', models.BigIntegerField(default=0, help_text='Bin containing values from 30% to 39%')),
                ('b4', models.BigIntegerField(default=0, help_text='Bin containing values from 40% to 49%')),
                ('b5', models.BigIntegerField(default=0, help_text='Bin containing values from 50% to 59%')),
                ('b6', models.BigIntegerField(default=0, help_text='Bin containing values from 60% to 69%')),
                ('b7', models.BigIntegerField(default=0, help_text='Bin containing values from 70% to 79%')),
                ('b8', models.BigIntegerField(default=0, help_text='Bin containing values from 80% to 89%')),
                ('b9', models.BigIntegerField(default=0, help_text='Bin containing values from 90% to 100%')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('instrument', models.ForeignKey(help_text='Related Instrument.', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.Instrument')),
                ('learner', models.ForeignKey(help_text='Related Learner.', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.Learner')),
            ],
            options={
                'unique_together': {('learner', 'instrument')},
            },
        ),
        migrations.CreateModel(
            name='HistogramActivityProvider',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('b0', models.BigIntegerField(default=0, help_text='Bin containing values from 0 to 9%')),
                ('b1', models.BigIntegerField(default=0, help_text='Bin containing values from 10% to 19%')),
                ('b2', models.BigIntegerField(default=0, help_text='Bin containing values from 20% to 29%')),
                ('b3', models.BigIntegerField(default=0, help_text='Bin containing values from 30% to 39%')),
                ('b4', models.BigIntegerField(default=0, help_text='Bin containing values from 40% to 49%')),
                ('b5', models.BigIntegerField(default=0, help_text='Bin containing values from 50% to 59%')),
                ('b6', models.BigIntegerField(default=0, help_text='Bin containing values from 60% to 69%')),
                ('b7', models.BigIntegerField(default=0, help_text='Bin containing values from 70% to 79%')),
                ('b8', models.BigIntegerField(default=0, help_text='Bin containing values from 80% to 89%')),
                ('b9', models.BigIntegerField(default=0, help_text='Bin containing values from 90% to 100%')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('activity', models.ForeignKey(help_text='Related Activity.', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.Activity')),
                ('provider', models.ForeignKey(help_text='Related Provider.', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.Provider')),
            ],
            options={
                'unique_together': {('activity', 'provider')},
            },
        ),
        migrations.CreateModel(
            name='HistogramActivityInstrument',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('b0', models.BigIntegerField(default=0, help_text='Bin containing values from 0 to 9%')),
                ('b1', models.BigIntegerField(default=0, help_text='Bin containing values from 10% to 19%')),
                ('b2', models.BigIntegerField(default=0, help_text='Bin containing values from 20% to 29%')),
                ('b3', models.BigIntegerField(default=0, help_text='Bin containing values from 30% to 39%')),
                ('b4', models.BigIntegerField(default=0, help_text='Bin containing values from 40% to 49%')),
                ('b5', models.BigIntegerField(default=0, help_text='Bin containing values from 50% to 59%')),
                ('b6', models.BigIntegerField(default=0, help_text='Bin containing values from 60% to 69%')),
                ('b7', models.BigIntegerField(default=0, help_text='Bin containing values from 70% to 79%')),
                ('b8', models.BigIntegerField(default=0, help_text='Bin containing values from 80% to 89%')),
                ('b9', models.BigIntegerField(default=0, help_text='Bin containing values from 90% to 100%')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('activity', models.ForeignKey(help_text='Related Activity.', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.Activity')),
                ('instrument', models.ForeignKey(help_text='Related Instrument.', on_delete=django.db.models.deletion.CASCADE, to='tesla_ce.Instrument')),
            ],
            options={
                'unique_together': {('activity', 'instrument')},
            },
        ),
    ]
