# Generated by Django 3.2.4 on 2021-06-16 18:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tesla_ce', '0014_uioption'),
    ]

    operations = [
        migrations.AddField(
            model_name='uioption',
            name='roles',
            field=models.CharField(blank=True, help_text='Required roles.', max_length=250, null=True),
        ),
    ]
