# Generated by Django 3.0.8 on 2020-07-30 09:50

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('tesla_ce', '0008_auto_20200721_1022'),
    ]

    operations = [
        migrations.CreateModel(
            name='Monitor',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.IntegerField(choices=[(0, 'Unknown'), (1, 'Ready'), (2, 'Error'), (3, 'Warning')], default=0, help_text='Status of the resource')),
                ('type', models.IntegerField(choices=[(0, 'Custom'), (1, 'Queue'), (2, 'Service'), (3, 'Provider'), (4, 'Instrument')], default=0, help_text='Type of the resource')),
                ('key', models.CharField(default=None, help_text='key of the resource', max_length=15, null=True)),
                ('info', models.TextField(help_text='Resource information.')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
